@@ -4,6 +4,7 @@
  * @author OpenCV team
  */
 
+#include "opencv2/imgcodecs.hpp"
 #include "opencv2/highgui/highgui.hpp"
 #include <iostream>
 
@@ -31,11 +32,11 @@ int main( void )
      { alpha = input; }
 
    /// Read image ( same size, same type )
-   src1 = imread("../images/LinuxLogo.jpg");
-   src2 = imread("../images/WindowsLogo.jpg");
+   src1 = imread("../data/LinuxLogo.jpg");
+   src2 = imread("../data/WindowsLogo.jpg");
 
-   if( !src1.data ) { std::cout<< "Error loading src1"<<std::endl; return -1; }
-   if( !src2.data ) { std::cout<< "Error loading src2"<<std::endl; return -1; }
+   if( src1.empty() ) { std::cout<< "Error loading src1"<<std::endl; return -1; }
+   if( src2.empty() ) { std::cout<< "Error loading src2"<<std::endl; return -1; }
 
    /// Create Windows
    namedWindow("Linear Blend", 1);
